unit UDisplay;

interface

uses
  Windows, Messages,  Classes, Controls, Forms,
  StdCtrls, FileCtrl, MMSystem, ExtCtrls, ShellApi, Menus;

const wm_IconMessage = WM_USER + 1;

type
  TDirectoryGuardFRM = class(TForm)
    AddBTN: TButton;
    ThreadsLBX: TListBox;
    PathLBX: TDirectoryListBox;
    DriveCBX: TDriveComboBox;
    DeleteBTN: TButton;
    EventsMMO: TMemo;
    ClearBTN: TButton;
    PathLBL: TLabel;
    ThreadsLBL: TLabel;
    EventsLBL: TLabel;
    Timer: TTimer;
    PopupMenu1: TPopupMenu;
    pmShow: TMenuItem;
    N1: TMenuItem;
    pmExit: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure AddBTNClick(Sender: TObject);
    procedure DeleteBTNClick(Sender: TObject);
    procedure ClearBTNClick(Sender: TObject);
    procedure WatchDone(Sender:TObject);
    procedure ThreadTerminate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure pmShowClick(Sender: TObject);
    procedure pmExitClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure IconTray(var MSG:TMessage);message wm_IconMessage;
  public
    { Public declarations }
    FInternalDriveChange:boolean;
    FIconState:boolean;
    FInitialized:boolean;
    nid : TNotifyIconData;
  end;

implementation

{$R *.DFM}
{$R *.RES}

uses UGuardThread,UInit,SysUtils,IniFiles;

procedure TDirectoryGuardFRM.FormCreate(Sender: TObject);
begin
  with nid do
  begin
    cbSize:=sizeOf(nid);
    wnd:=Handle;
    uID:=1;
    uCallbackMessage:=wm_IconMessage;
    hIcon:=Ic1.Handle;
    szTip:='Directory Guard';
    uFlags:=nif_Message or nif_Icon or nif_Tip;
    Shell_NotifyIcon(NIM_ADD,@nid);
  end;
end;

procedure TDirectoryGuardFRM.FormShow(Sender: TObject);
var FIniFile:TIniFile;
    lvs:string;
begin
  if not FInitialized then
  Begin
    FIniFile:=TIniFile.Create(ExtractFilePath(ParamStr(0))+'DGuard.ini');
    try
      DriveCBX.Drive:=FIniFile.ReadString('InitDir','Drive','C')[1];
      PathLBX.Directory:=DriveCBX.Drive+':\'+FIniFile.ReadString('InitDir','Dir','');
      lvs:=FIniFile.ReadString('InitDir','Index','Last');
      if lvs = '<Last>' then
        PathLBX.ItemIndex:=PathLBX.Items.Count-1
      else
      if lvs = '<First>' then
        PathLBX.ItemIndex:=0
      else
        PathLBX.ItemIndex:=PathLBX.Items.IndexOf(lvs);
      if PathLBX.ItemIndex>-1 then
        PathLBX.Selected[PathLBX.ItemIndex];
    finally
      FIniFile.Free;
    end;
    FInitialized:=True;
  End;
  ShowWindow(Handle,sw_ShowNormal);
  SetForegroundWindow(Handle);
  Timer.Enabled:=False;
  FIconState:=True;
  TimerTimer(Timer);
end;

procedure TDirectoryGuardFRM.AddBTNClick(Sender: TObject);
var cv:integer;
begin
  if FlagsForm.ShowModal=mrOK then
  with TDirectoryGuardThread(ThreadsLBX.Items.Objects[
        ThreadsLBX.Items.AddObject(PathLBX.Directory,TDirectoryGuardThread.Create(WatchDone))]) do
  Begin
    Path:=PathLBX.Directory;
    Name:=PathLBX.Directory;
    for cv:=1 to FlagsForm.FlagsCLB.Items.Count-1 do
      if FlagsForm.FlagsCLB.Checked[cv] then
        Flags:=Flags or (1 shl (cv-1));
    SubTree:=FlagsForm.FlagsCLB.Checked[0];
    OnTerminate:=ThreadTerminate;
    Resume;
  End;
end;

procedure TDirectoryGuardFRM.DeleteBTNClick(Sender: TObject);
begin
  if ThreadsLBX.ItemIndex>-1 then
  try
    TDirectoryGuardThread(ThreadsLBX.Items.Objects[ThreadsLBX.ItemIndex]).Terminate;
  except
  End;
end;

procedure TDirectoryGuardFRM.ClearBTNClick(Sender: TObject);
begin
  EventsMMO.Clear;
end;

procedure TDirectoryGuardFRM.WatchDone(Sender:TObject);
begin
  try
    case (Sender as TDirectoryGuardThread).Status of
      WAIT_OBJECT_0:
        Begin
          EventsMMO.Lines.Add((Sender as TDirectoryGuardThread).Name+' ('+DateTimeToStr(Now)+')');
          if not Timer.Enabled then
            Timer.Enabled:=not Visible;
        End;
      WAIT_TIMEOUT:
        Begin
        End;
      else
        Begin
          EventsMMO.Lines.Add('Guarding ERROR('+ IntToStr(Ord((Sender as TDirectoryGuardThread).Status))+') at '+
             (Sender as TDirectoryGuardThread).Name+' directory ('+DateTimeToStr(Now)+')'+' TERMINATED.');
          (Sender as TDirectoryGuardThread).Terminate;
        End;
    End;
  except
  end;
end;

procedure TDirectoryGuardFRM.ThreadTerminate(Sender: TObject);
Begin
  try
    ThreadsLBX.Items.Delete(ThreadsLBX.Items.IndexOf(TDirectoryGuardThread(Sender).Name));
  except
  end;
End;

procedure TDirectoryGuardFRM.TimerTimer(Sender: TObject);
begin
  if FIconState then
  Begin
    PlaySound('ALERT',HInstance,SND_RESOURCE or SND_ASYNC);
    nid.hIcon:=Ic1.Handle
  End
  else
    nid.hIcon:=Ic2.Handle;
  nid.uFlags := nif_Icon or nif_Tip;
  Shell_NotifyIcon(NIM_MODIFY, @nid);
  FIconState:=not FIconState;
end;

procedure TDirectoryGuardFRM.pmShowClick(Sender: TObject);
begin
  //ShowWindow(Handle,sw_ShowNormal);
  SetForegroundWindow(Handle);
  Show;
end;

procedure TDirectoryGuardFRM.pmExitClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TDirectoryGuardFRM.IconTray(var MSG: TMessage);
var Pt:TPoint;
begin
  case MSG.LParam of
    wm_RBUTTONDOWN:
      begin
        GetCursorPos(Pt);
        SetForegroundWindow(Handle);
        PopupMenu1.Popup(Pt.x,Pt.Y);
      end;
    wm_LBUTTONDBLCLK:
        pmShowClick(Self);
  end;
end;

procedure TDirectoryGuardFRM.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caNone;
  Hide;
end;

procedure TDirectoryGuardFRM.FormDestroy(Sender: TObject);
var cv:integer;
begin
  try
    if ThreadsLBX.Items.Count>0 then
      for cv:=ThreadsLBX.Items.Count-1 downto 0 do
      Begin
        TDirectoryGuardThread(ThreadsLBX.Items.Objects[cv]).Terminate;
        TDirectoryGuardThread(ThreadsLBX.Items.Objects[cv]).WaitFor;
      End;
  except
  end;
  nid.uFlags:=0;
  Shell_NotifyIcon(NIM_DELETE,@nid);
end;

end.
