unit UGuardThread;

interface

uses
  Classes,Windows;

type

  TDirectoryGuardThread = class(TThread)
  private
    { Private declarations }
  protected
    FEvent:TNotifyEvent;
    FName:string;
    FPath:string;
    FFlags:cardinal;
    FTimeOut:integer;
    FStatus:HResult;
    FSubTree:boolean;
    FChangeHandle:THandle;
    procedure Execute; override;
    procedure InternalDone;
  public
    constructor Create(ADoOnTerm:TNotifyEvent);
    property Path:String read FPath write FPath;
    property Flags:cardinal read FFlags write FFlags;
    property Name:string read FName write FName;
    property Status:HResult read FStatus;
    property SubTree:boolean read FSubTree write FSubTree;
    destructor Destroy; override;
  end;

  const GUARD_ERROR = 10010;
implementation

constructor TDirectoryGuardThread.Create(ADoOnTerm:TNotifyEvent);
Begin
  inherited Create(True);
  FEvent:=ADoOnTerm;
  FreeOnTerminate:=True;
  FTimeOut:=500;
  FFlags:=0;
End;

procedure TDirectoryGuardThread.Execute;
begin
  FChangeHandle:=FindFirstChangeNotification(PChar(FPath),FSubTree,FFlags);
  if FChangeHandle=INVALID_HANDLE_VALUE then
  Begin
    FStatus:=GUARD_ERROR;
    Exit;
  End;
  FStatus:=WaitForSingleObjectEx(FChangeHandle,FTimeOut,true);
  Synchronize(InternalDone);
  While not Terminated do
  Begin
    if FStatus=WAIT_OBJECT_0 then
      if not FindNextChangeNotification(FChangeHandle) then
      Begin
        FStatus:=GUARD_ERROR;
        break;
      End;
    FStatus:=WaitForSingleObjectEx(FChangeHandle,FTimeOut,false);
    Synchronize(InternalDone);
  End;
  FindCloseChangeNotification(FChangeHandle);
  if Assigned(FEvent) and (FStatus=GUARD_ERROR) then
  Begin
    FStatus:=GetLastError;
    Synchronize(InternalDone);  
  End;
end;

procedure TDirectoryGuardThread.InternalDone;
Begin
  if (not Terminated) and assigned(FEvent) then
  try
    FEvent(Self);
  except
  end;
End;

destructor TDirectoryGuardThread.Destroy;
Begin
  inherited;
End;

end.
